from dojango.forms.widgets   import TextInput
from dojango.forms.fields      import DojoFieldMixin

from django.utils.encoding   import force_unicode
from django.utils.safestring import mark_safe
from django.forms.util       import flatatt
from django.forms.fields     import Field
from django.forms.util       import ValidationError

#WIDGET
class SelectAjax(TextInput):

    """A combobox that is receiving data from a given dojo data url.
    As default dojo.data.ItemFileReadStore is used.
    Usage:
        SelectAjax (  url, model, parent, child, identifier, label, alias, attrs, store, store_attrs )
    """
    dojo_type = 'dijit.form.FilteringSelect'

    valid_extra_attrs = [
        'required',
        'help_text',
    ]


    def __init__( self, url, model, parent = None, child = None,
                  identifier='id', label='name', alias='value',
                  attrs= None, store=None, store_attrs={}):

        self.url        = url
        self.model      = model
        self.identifier = identifier
        self.label      = label
        self.alias      = alias
        self.parent     = parent
        self.child      = child

        if store:
            self.store = store
        else:
            self.store = 'dojo.data.ItemFileReadStore'

        if store_attrs:
            self.store_attrs = store_attrs
        else:
            self.store_attrs = {}

        self.extra_dojo_require.append(self.store)
        super(SelectAjax, self).__init__(attrs)


    def render(self, name, value, attrs=None):

        jsScript, jsStore, jsOnClick, jsOnChange, jsOnLoad, qs = '', '', '', '', '', None

        if value is None: value = ''

        store_id = self.get_store_id(getattr(attrs, "id", None), name)

        store_attrs = self.store_attrs.copy()

        final_attrs = self.build_attrs ( attrs,
                                         type=self.input_type,
                                         name=name,
                                         store=store_id)

        #se existir child sempre tera onChange (insert or update)
        if self.child: # and self.url
            jsOnChange = "dojo.connect( obj , 'onChange', function(){var oCaller = dijit.byId( '%(id)s' );" + \
                         "var oChild = dijit.byId( '%(child)s' );var sUrl = oChild.attr('url') + oCaller.value + '/';" + \
                         "oChild.store = new dojo.data.ItemFileReadStore({url:sUrl});oChild.attr('displayedValue', ''); " + \
                         "oChild.store.fetch({onBegin:function(){dijit.byId( '%(child)s' ).attr('displayedValue', 'carregando...')}, " + \
                         "onComplete: function(){ dijit.byId( '%(child)s' ).attr('displayedValue', '')}});" + \
                         "});"

        # TODO : Improve the feedback (onBegin, OnComplete)

        if self.url:
            jsOnLoad = "obj.attr('url','%(url)s');"
        else:
            qs = self.model.objects.all()

        #rotina quando objeto for para update
        if value != '':
            final_attrs['value'] = force_unicode(value)

            if not qs:
                qs = self.model.objects.filter(pk=value)

            #se tiver parent tera onClick somente quando update
            if self.parent:
                jsOnClick = "dojo.connect(obj,'onClick',function(){var oCaller = dijit.byId( '%(id)s' );" + \
                            "if ( oCaller.attr('loaded') == undefined ) {var sId = dijit.byId( '%(parent)s' ).value;" + \
                            "var sUrl = oCaller.attr('url') + sId + '/' ;oCaller.store = new dojo.data.ItemFileReadStore({url:sUrl});" + \
                            "oCaller.attr('loaded', sId);};});"

        if value == "" and self.url == "":
            store_attrs.update({'url': self.url,})

        if qs:
            jsStore = 'var _%s = %s;' % ( name, dojoComboData(self.identifier, self.label, qs, self.alias) )
            store_attrs.update({'data': ( '_%s' % name ) ,})

        jsAux = "<script>" + jsStore + "dojo.addOnLoad(function() { var obj = dijit.byId('%(id)s'); " + \
                jsOnLoad + jsOnClick + jsOnChange +  "})</script>"

        jsScript = jsAux  % {'name':name,
                             'url':self.url,
                             'id':attrs['id'],
                             'parent':self.parent,
                             'child':self.child }

        store_attrs.update({
            'dojoType': self.store,
            'jsId': store_id
        })

        store_node = '<div%s></div>' % flatatt(store_attrs)

        return mark_safe(u'%s%s<input%s />' % ( jsScript, store_node, flatatt(final_attrs)))


    def get_store_id(self, id, name):
        return "_store_" + (id and id or name)


#FIELD
class ModelChoiceFieldAjax(DojoFieldMixin, Field):

    def __init__( self,
                  model               ,
                  url                 = None,
                  child               = None,
                  parent              = None,
                  required            = None,
                  label               = None,
                  initial             = None,
                  help_text           = None,
                  error_messages      = None,
                  show_hidden_initial = None, *args, **kwargs):

        self.child = child
        self.parent = parent
        self.model = model
        self.url = url

        super(ModelChoiceFieldAjax, self).__init__ (    required,
                                                        SelectAjax(self.url, self.model, self.parent, self.child ),
                                                        label,
                                                        initial,
                                                        help_text,
                                                        error_messages,
                                                        show_hidden_initial )

    def clean(self, value):
        if value:
            try:
                value = int(value)
            except:
                raise ValidationError(self.error_messages['invalid_type'])
        return self.model(pk=value)

def dojoComboData ( identifier             ,
                    label                  ,
                    qs                     ,
                    identifier_alias = None,
                    label_alias      = None):
    
    items = ""
    
    if not identifier_alias:
        identifier_alias = identifier
        
    if not label_alias:
        label_alias = label
    
    for reg in qs:
        items += "{%s:'%i', %s:'%s'},"  % (identifier_alias,reg.__getattribute__(identifier), label_alias, reg.__getattribute__(label))
    data  = "{identifier:'%s',label:'%s',items:[%s]}" % ( identifier_alias, label_alias, items[:-1] )
    return data